# frozen_string_literal: true

module Hermes
  class HousingRequestEntityForm
    include Hermes::Concerns::CommonFieldsConcern

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    field :active, type: Boolean, default: false
    field :entity_id, type: BSON::ObjectId
    field :housing_request_id, type: BSON::ObjectId
    field :applicant_id, type: BSON::ObjectId
    field :doc_id, type: BSON::ObjectId
    field :consent_fields, type: Array, default: []
    field :level, type: Integer
    field :delegation, type: Boolean, default: false
    field :delegation_signed, type: Boolean, default: false

    belongs_to :entity_form_level, class_name: 'Hermes::HousingRequestEntityFormLevel', foreign_key: :entity_form_level_id

    validates_presence_of :housing_request_id

    common_indexes *%i[
      entity_id
      housing_request_id
      entity_form_level_id
      applicant_id
      doc_id
    ]

    before_save :before_save_actions
    after_save :after_save_actions

    private

    def set_applicant_id
      request = Hermes::HousingRequest.find_by(id: self.housing_request_id)
      self.applicant_id = request.main_applicant_id if request
    end

    def set_entity_id
      entity = Hermes::Entity.find_by(id: entity_form_level&.entity_id)
      self.set(entity_id: entity.id) if entity
    end

    def set_level
      self.set(level: entity_form_level&.level)
    end

    def notify_delegation_for_applicant
      if delegation_changed? && delegation_change.last
        housing_request = Hermes::HousingRequest.find_by(id: housing_request_id)
        if housing_request && !delegation_signed
          housing_request.notify(key: NOTIFICATION_KEYS[:housing_request][:delegation_signed_required], main_only: true)
        end
      end
    end

    def before_save_actions
      set_applicant_id if housing_request_id_changed?
    end

    def after_save_actions
      if entity_form_level_id_changed?
        set_level
        set_entity_id
      end
      notify_delegation_for_applicant
    end
  end
end